@@ -70,21 +70,7 @@ g_delayed_settings_backend_notify_unapplied (GDelayedSettingsBackend *delayed)
   g_static_mutex_unlock (&delayed->priv->lock);
 
   if (target != NULL)
-    {
-      if (g_settings_backend_get_active_context () != target_context)
-        {
-          GSource *source;
-
-          source = g_idle_source_new ();
-          g_source_set_priority (source, G_PRIORITY_DEFAULT);
-          g_source_set_callback (source, invoke_notify_unapplied,
-                                 target, g_object_unref);
-          g_source_attach (source, target_context);
-          g_source_unref (source);
-        }
-      else
-        invoke_notify_unapplied (target);
-    }
+    g_main_context_invoke (target_context, invoke_notify_unapplied, target);
 }
 
 
@@ -95,15 +81,29 @@ g_delayed_settings_backend_read (GSettingsBackend   *backend,
                                  gboolean            default_value)
 {
   GDelayedSettingsBackend *delayed = G_DELAYED_SETTINGS_BACKEND (backend);
-  GVariant *result;
+  gpointer result = NULL;
 
-  if (!default_value &&
-      (result = g_tree_lookup (delayed->priv->delayed, key)))
-    return g_variant_ref (result);
+  if (!default_value)
+    {
+      g_static_mutex_lock (&delayed->priv->lock);
+      if (g_tree_lookup_extended (delayed->priv->delayed, key, NULL, &result))
+        {
+          /* NULL in the tree means we should consult the default value */
+          if (result != NULL)
+            g_variant_ref (result);
+          else
+            default_value = TRUE;
+        }
+      g_static_mutex_unlock (&delayed->priv->lock);
+    }
 
-  return g_settings_backend_read (delayed->priv->backend,
-                                  key, expected_type, default_value);
+  if (result == NULL)
+    result = g_settings_backend_read (delayed->priv->backend, key,
+                                      expected_type, default_value);
+
+  return result;
 }
+
 static gboolean
 g_delayed_settings_backend_write (GSettingsBackend *backend,
                                   const gchar      *key,
@@ -137,7 +137,7 @@ add_to_tree (gpointer key,
 }
 
 static gboolean
-g_delayed_settings_backend_write_keys (GSettingsBackend *backend,
+g_delayed_settings_backend_write_tree (GSettingsBackend *backend,
                                        GTree            *tree,
                                        gpointer          origin_tag)
 {
@@ -172,15 +172,16 @@ g_delayed_settings_backend_reset (GSettingsBackend *backend,
                                   const gchar      *key,
                                   gpointer          origin_tag)
 {
-  /* deal with this... */
-}
+  GDelayedSettingsBackend *delayed = G_DELAYED_SETTINGS_BACKEND (backend);
+  gboolean was_empty;
 
-static void
-g_delayed_settings_backend_reset_path (GSettingsBackend *backend,
-                                       const gchar      *path,
-                                       gpointer          origin_tag)
-{
-  /* deal with this... */
+  g_static_mutex_lock (&delayed->priv->lock);
+  was_empty = g_tree_nnodes (delayed->priv->delayed) == 0;
+  g_tree_insert (delayed->priv->delayed, g_strdup (key), NULL);
+  g_static_mutex_unlock (&delayed->priv->lock);
+
+  if (was_empty)
+    g_delayed_settings_backend_notify_unapplied (delayed);
 }
 
 static void
@@ -231,7 +232,7 @@ g_delayed_settings_backend_apply (GDelayedSettingsBackend *delayed)
       g_static_mutex_lock (&delayed->priv->lock);
       tmp = delayed->priv->delayed;
       delayed->priv->delayed = g_settings_backend_create_tree ();
-      success = g_settings_backend_write_keys (delayed->priv->backend,
+      success = g_settings_backend_write_tree (delayed->priv->backend,
                                                tmp, delayed->priv);
       g_static_mutex_unlock (&delayed->priv->lock);
 
@@ -265,8 +266,8 @@ g_delayed_settings_backend_revert (GDelayedSettingsBackend *delayed)
 
 /* change notification */
 static void
-delayed_backend_changed (GSettingsBackend *backend,
-                         GObject          *target,
+delayed_backend_changed (GObject          *target,
+                         GSettingsBackend *backend,
                          const gchar      *key,
                          gpointer          origin_tag)
 {
@@ -278,8 +279,8 @@ delayed_backend_changed (GSettingsBackend *backend,
 }
 
 static void
-delayed_backend_keys_changed (GSettingsBackend    *backend,
-                              GObject             *target,
+delayed_backend_keys_changed (GObject             *target,
+                              GSettingsBackend    *backend,
                               const gchar         *path,
                               const gchar * const *items,
                               gpointer             origin_tag)
@@ -292,8 +293,8 @@ delayed_backend_keys_changed (GSettingsBackend    *backend,
 }
 
 static void
-delayed_backend_path_changed (GSettingsBackend *backend,
-                              GObject          *target,
+delayed_backend_path_changed (GObject          *target,
+                              GSettingsBackend *backend,
                               const gchar      *path,
                               gpointer          origin_tag)
 {
@@ -305,8 +306,8 @@ delayed_backend_path_changed (GSettingsBackend *backend,
 }
 
 static void
-delayed_backend_writable_changed (GSettingsBackend *backend,
-                                  GObject          *target,
+delayed_backend_writable_changed (GObject          *target,
+                                  GSettingsBackend *backend,
                                   const gchar      *key)
 {
   GDelayedSettingsBackend *delayed = G_DELAYED_SETTINGS_BACKEND (target);
@@ -314,11 +315,16 @@ delayed_backend_writable_changed (GSettingsBackend *backend,
 
   g_static_mutex_lock (&delayed->priv->lock);
 
-  if (g_tree_lookup (delayed->priv->delayed, key) &&
+  if (g_tree_lookup (delayed->priv->delayed, key) != NULL &&
       !g_settings_backend_get_writable (delayed->priv->backend, key))
     {
       /* drop the key from our changeset if it just became read-only.
        * no need to signal since the writable change below implies it.
+       *
+       * note that the item in the tree may very well be set to NULL in
+       * the case that the user stored a reset.  we intentionally don't
+       * drop the key in this case since a reset will always succeed
+       * (even against a non-writable key).
        */
       g_tree_remove (delayed->priv->delayed, key);
 
@@ -357,8 +363,8 @@ check_prefix (gpointer key,
 }
 
 static void
-delayed_backend_path_writable_changed (GSettingsBackend *backend,
-                                       GObject          *target,
+delayed_backend_path_writable_changed (GObject          *target,
+                                       GSettingsBackend *backend,
                                        const gchar      *path)
 {
   GDelayedSettingsBackend *delayed = G_DELAYED_SETTINGS_BACKEND (target);
@@ -377,9 +383,12 @@ delayed_backend_path_writable_changed (GSettingsBackend *backend,
       /* collect a list of possibly-affected keys (ie: matching the path) */
       g_tree_foreach (delayed->priv->delayed, check_prefix, &state);
 
-      /* drop the keys that have been affected */
+      /* drop the keys that have been affected.
+       *
+       * don't drop 'reset' keys (see above) */
       for (i = 0; i < state.index; i++)
-        if (!g_settings_backend_get_writable (delayed->priv->backend,
+        if (g_tree_lookup (delayed->priv->delayed, state.keys[i]) != NULL &&
+            !g_settings_backend_get_writable (delayed->priv->backend,
                                               state.keys[i]))
           g_tree_remove (delayed->priv->delayed, state.keys[i]);
 
@@ -423,9 +432,8 @@ g_delayed_settings_backend_class_init (GDelayedSettingsBackendClass *class)
 
   backend_class->read = g_delayed_settings_backend_read;
   backend_class->write = g_delayed_settings_backend_write;
-  backend_class->write_keys = g_delayed_settings_backend_write_keys;
+  backend_class->write_tree = g_delayed_settings_backend_write_tree;
   backend_class->reset = g_delayed_settings_backend_reset;
-  backend_class->reset_path = g_delayed_settings_backend_reset_path;
   backend_class->get_writable = g_delayed_settings_backend_get_writable;
   backend_class->subscribe = g_delayed_settings_backend_subscribe;
   backend_class->unsubscribe = g_delayed_settings_backend_unsubscribe;
@@ -462,6 +470,13 @@ g_delayed_settings_backend_new (GSettingsBackend *backend,
                                 gpointer          owner,
                                 GMainContext     *owner_context)
 {
+  static GSettingsListenerVTable vtable = {
+    delayed_backend_changed,
+    delayed_backend_path_changed,
+    delayed_backend_keys_changed,
+    delayed_backend_writable_changed,
+    delayed_backend_path_writable_changed
+  };
   GDelayedSettingsBackend *delayed;
 
   delayed = g_object_new (G_TYPE_DELAYED_SETTINGS_BACKEND, NULL);
@@ -471,12 +486,8 @@ g_delayed_settings_backend_new (GSettingsBackend *backend,
 
   g_object_weak_ref (owner, g_delayed_settings_backend_disown, delayed);
 
-  g_settings_backend_watch (delayed->priv->backend, G_OBJECT (delayed), NULL,
-                            delayed_backend_changed,
-                            delayed_backend_path_changed,
-                            delayed_backend_keys_changed,
-                            delayed_backend_writable_changed,
-                            delayed_backend_path_writable_changed);
+  g_settings_backend_watch (delayed->priv->backend,
+                            &vtable, G_OBJECT (delayed), NULL);
 
   return delayed;
 }
